package cn.jyjz.xiaoyao.common.redis.cache;


import cn.jyjz.xiaoyao.common.base.util.SpringUtils;
import cn.jyjz.xiaoyao.common.redis.cache.ehcache.EhCacheManagerImpl;
import cn.jyjz.xiaoyao.common.redis.cache.redis.RedisManagerImpl;
import cn.jyjz.xiaoyao.common.redis.cache.redis.RedisUtils;
import cn.jyjz.xiaoyao.common.redis.config.CacheProperties;
import cn.jyjz.xiaoyao.common.task.register.CronTaskRegistrar;
import cn.jyjz.xiaoyao.common.task.register.SchedulingRunnable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.cache.ehcache.EhCacheCacheManager;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.DependsOn;
import org.springframework.data.redis.cache.RedisCacheManager;

/**
 * Created by zhangyong on 2017/12/28.
 */
@Configuration
@AutoConfigureAfter({EhCacheCacheManager.class,EhCacheCacheManager.class})
@EnableConfigurationProperties(CacheProperties.class)
@DependsOn({"springUtils"})
public class CacheBeanConfig {

    private Logger logger = LoggerFactory.getLogger(CacheBeanConfig.class);
    @Autowired
    private CacheProperties cacheProperties;

    @Autowired
    CronTaskRegistrar cronTaskRegistrar;

    @Bean
    @ConditionalOnMissingBean(name = "iCacheManager")
    public ICacheManager iCacheManagerGenerator(){

        ICacheManager iCacheManager = null;

        EhCacheCacheManager ehCacheManager = SpringUtils.getBean(EhCacheCacheManager.class);
        RedisCacheManager redisCacheManager = SpringUtils.getBean(RedisCacheManager.class);

        if(null != ehCacheManager){
            logger.debug("使用EhCacheCache");

            //初始化缓存接口的实现对象
            EhCacheManagerImpl ehcacheGenerator = new EhCacheManagerImpl(ehCacheManager,cacheProperties);

            SchedulingRunnable task = new SchedulingRunnable("ehcacheTask", "ehcachetask",ehCacheManager);
            cronTaskRegistrar.addCronTask(task, "0/10 * * * * ?");

            iCacheManager = ehcacheGenerator;
        }
        else {
            if (null != redisCacheManager) {


                RedisUtils redisUtils = SpringUtils.getBean(RedisUtils.class);

                RedisManagerImpl redisManagerImpl = new RedisManagerImpl(redisUtils, cacheProperties);

                iCacheManager = redisManagerImpl;
                logger.debug("使用reid");
            }
        }

        return iCacheManager;
    }

}