package cn.jyjz.xiaoyao.common.redis.cache.ehcache;

import cn.jyjz.xiaoyao.common.redis.cache.ICacheManager;
import cn.jyjz.xiaoyao.common.redis.config.CacheProperties;
import com.alibaba.fastjson.JSONObject;
import java.util.ArrayList;
import java.util.List;
import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;
import net.sf.ehcache.statistics.StatisticsGateway;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.ehcache.EhCacheCacheManager;

/**
 * Created by zhangyong on 2018/2/27.
 */
public class EhCacheManagerImpl implements ICacheManager {

    private Logger log = LoggerFactory.getLogger(this.getClass());

    private EhCacheCacheManager ehCacheCacheManager;

    private CacheProperties cacheProperties;

    public EhCacheManagerImpl(org.springframework.cache.CacheManager cacheManager,CacheProperties cacheProperties) {
        ehCacheCacheManager = (EhCacheCacheManager) cacheManager;
        this.cacheProperties = cacheProperties;

    }

    /**
     * 获得一个Cache，没有则创建一个。
     *
     * @param cacheName 缓存名称
     * @param saveTime  存活时间
     * @param validTime 活动时间
     * @return 返回缓存对象
     */
    private Cache getCache(String cacheName, long saveTime, long validTime) {
        Cache cache = initCache(null, saveTime, cacheName, ehCacheCacheManager.getCacheManager(), validTime, false);

        return cache;
    }

    /**
     * 获得一个Cache，没有则创建一个。
     *
     * @param cacheName 缓存名称
     * @param validTime 活动时间
     * @return 返回缓存对象
     */
    private Cache getCache(String cacheName, long validTime) {
        Cache cache = initCache(null, -1, cacheName, ehCacheCacheManager.getCacheManager(), validTime, false);
        return cache;
    }

    /**
     * 获得一个Cache，没有则创建一个。
     *
     * @param cacheName 缓存名称
     * @return 返回缓存对象
     */
    private Cache getCache(String cacheName) {
        Cache cache = initCache(null, 1800, cacheName, ehCacheCacheManager.getCacheManager(), 1800, true);
        return cache;
    }

    /**
     * @param url       terracotta服务地址
     * @param saveTime  缓存存活时间
     * @param cachename 缓存名称
     * @param manager   缓存控制器
     * @param validTime 缓存活动超时时间
     * @param eternal   是否永久缓存
     * @return 返回cache对象
     */
    public Cache initCache(String url, long saveTime, String cachename, CacheManager manager, long validTime, boolean eternal) {

        Cache cache = manager.getCache(cachename);

        //缓存如果不存在，创建一个缓存
        if (null == cache) {
            cache = CacheUtileBase.initCache(url, saveTime, cachename, validTime, eternal);

            //缓存对象，初始化到缓存控制器中
            try {
                manager.addCache(cache);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        //cache.setca
        //cache.getCacheEventNotificationService().
        //List list = cache.getKeysWithExpiryCheck();
        return cache;
    }


//
//    /**
//     * 获得一个Cache，没有也不创建
//     *
//     * @param cacheName 缓存名称
//     * @return 返回缓存对象
//     */
//    private Cache getCacheNoInit(String cacheName) {
//        Cache cache = cacheManager.getCache(cacheName);
//
//        return cache;
//    }

    @Override
    public <T extends Object> T get(String cacheName, String key) {
        cacheName = cacheProperties.getPrefix() + cacheName;

        Cache cache = ehCacheCacheManager.getCacheManager().getCache(cacheName);

        Element element = null;
        if (null != cache) {
            try {
                element = cache.get(key);
            } catch (Exception e) {
                log.error("缓存获取错误：{}", e);
                element = null;
            }

        }

        return element == null ? null : (T) element.getObjectValue();
    }

    /**
     * 查询缓存
     * 根据key获取到缓存集合
     *
     * @return 返回缓存对象
     */
    @Override
    public <T extends Object> List<T> get(String cacheName) {
        cacheName = cacheProperties.getPrefix() + cacheName;

        List<T> listValue = null;
        Cache cache = ehCacheCacheManager.getCacheManager().getCache(cacheName);
        if (null != cache) {
            List list = cache.getKeys();
            listValue = new ArrayList<>();
            for (Object key : list) {
                Element element = cache.get(key);
                if (null != element) {
                    listValue.add((T) element.getObjectValue());
                }

            }
        }

        return listValue;
    }

    @Override
    public void put(String cacheName, String key, Object value) {
        cacheName = cacheProperties.getPrefix() + cacheName;

        //获取缓存对象
        Cache cache = getCache(cacheName);

        Element element = new Element(key, value);
        try {
            cache.put(element);
        } catch (Exception e) {
            log.error("缓存保存出错：{}", e);
        }
    }

    @Override
    public void put(String cacheName, String key, Object value, long validTime) {
        cacheName = cacheProperties.getPrefix() + cacheName;

        //获取缓存对象
        Cache cache = getCache(cacheName, validTime);

        Element element = new Element(key, value);
        try {
            cache.put(element);
        } catch (Exception e) {
            log.error("缓存保存出错：{}", e);
        }
    }

    /**
     * 添加缓存
     * 添加一个缓冲数据
     *
     * @param key       缓存key
     * @param value     缓冲的缓存数据
     * @param saveTime  存活时间
     * @param validTime 超时时间，存活时间
     */
    @Override
    public void put(String cacheName, String key, Object value, long saveTime, long validTime) {
        cacheName = cacheProperties.getPrefix() + cacheName;

        //获取缓存对象

        Cache cache = getCache(cacheName, saveTime, validTime);

        Element element = new Element(key, value);
        try {
            cache.put(element);
        } catch (Exception e) {
            log.error("缓存保存出错：{}", e);
        }

    }

    @Override
    public void remove(String cacheName, String key) {
        cacheName = cacheProperties.getPrefix() + cacheName;

        Cache cache = ehCacheCacheManager.getCacheManager().getCache(cacheName);
        if (null != cache) {
            cache.remove(key);
        }

    }

    @Override
    public void removeAll(String cacheName) {
        cacheName = cacheProperties.getPrefix() + cacheName;

        Cache cache = ehCacheCacheManager.getCacheManager().getCache(cacheName);
        if (null != cache) {
            cache.removeAll(true);
        }
    }

    @Override
    public JSONObject getCacheInfo() {

        JSONObject obj = new JSONObject();

        CacheManager cacheManager = ehCacheCacheManager.getCacheManager();

        String[] names = cacheManager.getCacheNames();

        long tempMemory = 0;
        for(String name:names){
            Cache tempCache = cacheManager.getCache(name);

            StatisticsGateway statisticsGateway = tempCache.getStatistics();
            tempMemory = tempMemory + statisticsGateway.getLocalHeapSizeInBytes();
        }

        obj.put("cachename","ehcache");
        obj.put("memory",(tempMemory/1024/1024) + "MB / " + (Runtime.getRuntime().totalMemory()/1024/1024) + "MB");

        return obj;
    }

}