package cn.jyjz.xiaoyao.common.redis.cache.redis.listener;

import cn.jyjz.xiaoyao.common.redis.cache.CacheListenerIngerface;
import cn.jyjz.xiaoyao.common.redis.cache.constants.CacheConstants;
import cn.jyjz.xiaoyao.common.redis.config.CacheProperties;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.listener.KeyExpirationEventMessageListener;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.stereotype.Component;

/**
 * @title RedisKeyExListener
 * 修改redis 的配置文件 notify-keyspace-events （""）Ex
 * 描述 监听redis中已经过期的key
 * Created by zhangyong on 2021/6/7.
 */
@Component
@ConditionalOnClass({RedisMessageListenerContainer.class})
@ConditionalOnProperty(prefix = "spring.cache", name = "type", havingValue = "redis", matchIfMissing = false)
public class RedisKeyExListener extends KeyExpirationEventMessageListener {

    @Autowired
    private CacheProperties cacheProperties;

    @Autowired
    private CacheListenerIngerface cacheListenerIngerface;

    public RedisKeyExListener(RedisMessageListenerContainer listenerContainer) {
        super(listenerContainer);
    }
    @Override
    public void onMessage(Message message, byte[] pattern) {
        String expiredKey = message.toString();

        String tempname = (cacheProperties.getPrefix() + "{" + CacheConstants.USER_MODEL_NAME);

        //TODO 需要测试
        //判断监听的主键是否为用户信息
        if(expiredKey.indexOf(tempname) >= 0){

            String token = expiredKey.substring(expiredKey.lastIndexOf("}")+1);

            if(token.indexOf(cacheProperties.getCacheNamePrix() + cacheProperties.getCacheNameTimePrix()) > 0){

                token = token.substring(0,token.lastIndexOf(cacheProperties.getCacheNamePrix() + cacheProperties.getCacheNameTimePrix()));
            }

            //删除在线用户表信息
            cacheListenerIngerface.removeCache(token);

        }
    }
}